require 'sinatra'
require 'json'

require_relative 'evidence'

class AnotherPlace < Sinatra::Base
  # set up our application's configuration
  configure do
    enable :logging
  end

  # set specific configurations to run before each request is handled
  before do
    logger.datetime_format = ''
    headers({"Content-Type" => "application/json"})
  end

  # set up a special 404 not found message
  not_found do
    status 404
    body("You're route (HTTP verb and path) didn't get you where you wanted to be...")
  end

  # create a special error handler
  class CustomClueError < Exception; end
  error CustomClueError do
    "Read your errors!\n" + env['sinatra.error'].message
  end

  get '/' do
    status 200
    body("::psst:: *check the headers, watch the logs...*\n")
  end

  get '/double_r_diner' do
    # STEP 1
    logger.info(Evidence::CLUES[request.path][:log_clue])
  end

  get '/pies' do
    if params[:kind] == 'cherry'
      # STEP 3
      logger.info(Evidence::CLUES[request.path][1][:log_clue])
      headers({
        "X-Clue" => Evidence::CLUES[request.path][1][:header_clue]
      })
      body(Evidence::CLUES[request.path][1][:body_clue].to_json)
    else
      # STEP 2
      headers({
        "X-Clue" => Evidence::CLUES[request.path][0][:header_clue]
      })
      body(Evidence::CLUES[request.path][0][:body_clue])
    end
  end

  get '/scenes/:scene' do
    if params[:scene] == 'crime'
      if params[:investigate] == 'body'
        # STEP 4
        logger.info(Evidence::CLUES[request.path][:log_clue])
        redirect to("/aXQgd2FzIHBq", headers({"Content-Transfer-Encoding" => 'base64'}))
      else
        logger.info("Right place... What do you want to *investigate*?")
      end
    else
      logger.info("Wrong scene... What are you here to investigate?")
    end
  end

  get '/aXQgd2FzIHBq' do
    # STEP 5
    logger.info(Evidence::CLUES[request.path][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][:header_clue],
      "Content-Transfer-Encoding" => 'base64'
    })
    body(Evidence::CLUES[request.path][:body_clue].to_json)
  end

  get '/aW4gdGhlIHJvb3QgZGlyZWN0b3J5' do
    # STEP 6
    logger.info(Evidence::CLUES[request.path][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][:header_clue]
    })
    body(Evidence::CLUES[request.path][:body_clue])
  end

  get '/alarm' do
    # STEP 7.1
    logger.info(Evidence::CLUES[request.path][0][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][0][:header_clue]
    })
    body(Evidence::CLUES[request.path][0][:body_clue])
  end

  delete '/alarm' do
    # STEP 7.2
    logger.info(Evidence::CLUES[request.path][1][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][1][:header_clue]
    })
    body(Evidence::CLUES[request.path][1][:body_clue])
  end

  put '/coffee' do
    # STEP 8
    if params[:extra].include?("sugar") && params[:extra].include?("cream")
      logger.info(Evidence::CLUES[request.path][:log_clue])
      headers({
        "X-Clue" => Evidence::CLUES[request.path][:header_clue]
      })
      body(Evidence::CLUES[request.path][:body_clue].to_json)
    else
      logger.info("Send the data: {'extra' => ['sugar','cream']}...")
    end
  end

  post '/police' do
    # STEP 9.1
    if params[:clue] == 'it_was_all_a_dream'
      logger.info(Evidence::CLUES[request.path][0][:log_clue])
      redirect to('/police', headers({"X-Clue" => Evidence::CLUES[request.path][0][:header_clue]}))
    end
  end

  get '/police' do
    # STEP 9.2
    logger.info(Evidence::CLUES[request.path][1][:header_clue])
    raise CustomClueError, Evidence::CLUES[request.path][1][:log_clue]
  end

  get '/another_place' do
    # STEP 10
    logger.info(Evidence::CLUES[request.path][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][:header_clue],
      "Content-Transfer-Encoding" => 'base64'
    })
    body(Evidence::CLUES[request.path][:body_clue].to_json)
  end

  get '/d2l0aCB0aGUgcm0gLXJm' do
    # THE FINAL CLUE
    logger.info(Evidence::CLUES[request.path][:log_clue])
    headers({
      "X-Clue" => Evidence::CLUES[request.path][:header_clue],
      "Content-Transfer-Encoding" => 'base64'
    })
    body(Evidence::CLUES[request.path][:body_clue].to_json)
  end
end

# Run the app!
#
puts "Starting a server in another place."
Evidence.print_to_log Evidence::START_MESSAGE

AnotherPlace.run!
